
######################################################################
# Modified SimpleXMLRPCServer to pass client address to called functions.
# Address is passed to function in clientAddr named argument, if the function takes it.
# Author: Petteri Aimonen <jpa@kapsi.fi>
# Licenced under GNU General Public Licence
# Part of the KBlackBoard distribution


from SimpleXMLRPCServer import *

class XMLRPCRequestHandler( SimpleXMLRPCRequestHandler ):
	def do_POST(self):
		"""Handles the HTTP POST request.
		
		Attempts to interpret all HTTP POST requests as XML-RPC calls,
		which are forwarded to the server's _dispatch method for handling.
		"""
		
		try:
			# get arguments
			data = self.rfile.read(int(self.headers["content-length"]))
			
			response = self.server._marshaled_dispatch(
				data, clientAddr = self.address_string()
			)
		except: # This should only happen if the module is buggy
			# internal error, report as HTTP server error
			self.send_response(500)
			self.end_headers()
		else:
			# got a valid XML RPC response
			self.send_response(200)
			self.send_header("Content-type", "text/xml")
			self.send_header("Content-length", str(len(response)))
			self.end_headers()
			self.wfile.write(response)
		
			# shut down the connection
			self.wfile.flush()
			self.connection.shutdown(1)


class XMLRPCServer( SimpleXMLRPCServer ):
	def __init__(self, addr, requestHandler=XMLRPCRequestHandler,logRequests=1):
		self.logRequests = logRequests

		SimpleXMLRPCDispatcher.__init__(self)
		SocketServer.TCPServer.__init__(self, addr, requestHandler)
	
	
	def _marshaled_dispatch(self, data, clientAddr = None):
		"""Dispatches an XML-RPC method from marshalled (XML) data.
		
		XML-RPC methods are dispatched from the marshalled (XML) data
		using the _dispatch method and the result is returned as
		marshalled data.
		"""
		
		params, method = xmlrpclib.loads(data)
		
		# generate response
		try:
			response = self._dispatch(method, params, clientAddr)
			# wrap response in a singleton tuple
			response = (response,)
			response = xmlrpclib.dumps(response, methodresponse=1, allow_none = True)
		except Fault, fault:
			response = xmlrpclib.dumps(fault)
		except:
			# report exception back to server
			response = xmlrpclib.dumps(
			xmlrpclib.Fault(1, "%s:%s" % (sys.exc_type, sys.exc_value))
			)
		
		return response
	
	def _dispatch(self, method, params, clientAddr):
		"""Dispatches the XML-RPC method.
	
		XML-RPC calls are forwarded to a registered function that
		matches the called XML-RPC method name. If no such function
		exists then the call is forwarded to the registered instance,
		if available.
	
		If the registered instance has a _dispatch method then that
		method will be called with the name of the XML-RPC method and
		it's parameters as a tuple
		e.g. instance._dispatch('add',(2,3))
	
		If the registered instance does not have a _dispatch method
		then the instance will be searched to find a matching method
		and, if found, will be called.
	
		Methods beginning with an '_' are considered private and will
		not be called.
		"""
	
		func = None
		try:
			# check to see if a matching function has been registered
			func = self.funcs[method]
		except KeyError:
			if self.instance is not None:
				# check for a _dispatch method
				if hasattr(self.instance, '_dispatch'):
					return self.instance._dispatch(method, params)
				else:
					# call instance method directly
					try:
						func = resolve_dotted_attribute(
						self.instance,
						method,
						self.allow_dotted_names
						)
					except AttributeError:
						pass
	
		if func is not None:
			try:
				return func(clientAddr = clientAddr, *params)
			except TypeError:
				return func(*params)
		else:
			raise Exception('method "%s" is not supported' % method)
	
